/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.act.utils;

import com.fasterxml.jackson.annotation.JsonBackReference;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.jeeplus.common.annotation.FieldName;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.utils.Encodes;
import com.jeeplus.common.utils.ObjectUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.modules.act.entity.Act;
import com.jeeplus.modules.sys.entity.Role;
import com.jeeplus.modules.sys.entity.SysRoleActivity;
import com.jeeplus.modules.sys.entity.User;
import org.activiti.engine.impl.persistence.entity.GroupEntity;
import org.activiti.engine.impl.persistence.entity.UserEntity;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 流程工具
 * @author jeeplus
 * @version 2013-11-03
 */
public class ActUtils {

//	private static Logger logger = LoggerFactory.getLogger(ActUtils.class);
	
	/**
	 * 定义流程定义KEY，必须以“PD_”开头
	 * 组成结构：string[]{"流程标识","业务主表表名"}
	 */
	public static final String[] PD_LEAVE = new String[]{"leave", "oa_leave"};
	public static final String[] PD_ALLAPPROVE = new String[]{"allApprove", "oa_all"};
	public static final String[] PD_TEST_AUDIT = new String[]{"test_audit", "oa_test_audit"};
	public static final String[] PD_OABUY = new String[]{"oaBuy", "oa_buy"};//采购申请
	public static final String[] PD_OAPERFORMANCE = new String[]{"oaPerformance", "oa_performance"};//绩效自评
	public static final String[] PD_EXPENSE_ACCOUNT = new String[]{"expense_account", "oa_expense_account"};//网上报销流程
	public static final String[] PD_NETWORK_SERVICE = new String[]{"network_service", "oa_network_service"};//网络报修流程
	public static final String[] PD_REIMBURSEMENT = new String[]{"reimbursement", "work_reimbursement"};
	public static final String[] PD_WORKFULLREOCRD = new String[]{"workfullrecord", "work_full_record"}; //项目归档
	public static final String[] PD_ENGINEERINGRECORD = new String[]{"engineeringrecord", "work_full_record"}; //项目归档
	public static final String[] PD_WORKFULLPAY = new String[]{"workfullpay", "work_full_pay"}; //工程计量支付
	public static final String[] PD_INVOICE = new String[]{"invoice", "work_invoice"};
	public static final String[] PD_OVERTIMEFORM = new String[]{"overtimeform","work_overtime_form"};
	public static final String[] PD_GO_OUT = new String[]{"goout","go_out"};
	public static final String[] PD_SEALFORM = new String[]{"sealform","work_seal_form"};
	public static final String[] PD_OA_EVECTION = new String[]{"evection","oa_evection"};
	public static final String[] PD_OA_PETTYCASH = new String[]{"pettyCash","work_petty_cash"};
	public static final String[] PD_OA_PAYMENT = new String[]{"payment","work_payment"};
	public static final String[] PD_OA_REIMBUR = new String[]{"reimbur","work_reimbur"};
	public static final String[] PD_JOIN_COMPANY = new String[]{"joincompany","sys_company_info"};
	public static final String[] PD_AUDIT_PROCESS1 = new String[]{"auditprocess1","sys_audit_process"};
	public static final String[] PD_AUDIT_PROCESS2 = new String[]{"auditprocess2","sys_audit_process"};
	public static final String[] PD_AUDIT_PROCESS3 = new String[]{"auditprocess3","sys_audit_process"};
	public static final String[] PD_AUDIT_PROCESS4 = new String[]{"auditprocess4","sys_audit_process"};
	public static final String[] PD_AUDIT_PROCESS5 = new String[]{"auditprocess5","sys_audit_process"};
	public static final String[] PD_WORKCONTRACT = new String[]{"contractAudit", "work_contract_info"};
	public static final String[] PD_CONTRACTINVALIDATE = new String[]{"contractInvalidate", "work_contract_info"};
	public static final String[] PD_DEPOSIT = new String[]{"deposit", "project_deposit_management"};
	public static final String[] PD_PREQUALIFICATION = new String[]{"prequalification", "work_bid_project"};
	public static final String[] PD_WORKBIDINGDOCUMENT = new String[]{"workBidingDocument","work_biding_document"};//标书编制
	public static final String[] PD_PROJECTALLOCATION = new String[]{"projectallocation","work_engineering_project"};
	public static final String[] PD_WORKINVOICE = new String[]{"workinvoice","work_invoice"};
	public static final String[] PD_WORKINVOICECANCLE = new String[]{"workinvoicecancle","work_invoice"};
	public static final String[] PD_WORKFULLEXECUTE  = new String[]{"workFullExecute","work_full_execute"};//项目实施
	public static final String[] PD_THREE_EXAMINATION  = new String[]{"threeexamination","work_engineering_project"};
	public static final String[] PD_WORKFULLMEETINGMINUTES = new String[]{"workFullMeetingminutes","work_full_meetingminutes"};//会议纪要
	public static final String[] PD_WORKFULLDESIGNCHANGE = new String[]{"workFullDesignchange","work_full_designchange"};//设计变更
	public static final String[] PD_WORKFULLCONSTRUCTSHEET = new String[]{"workFullConstructsheet","work_full_constructsheet"};//施工联系单
	public static final String[] PD_WORKFULLPROPRIETORSHEET = new String[]{"workFullProprietorsheet","work_full_proprietorsheet"};//业主联系单
	public static final String[]  PD_WORKFULLSUPERVISORSHEET = new String[]{"workFullSupervisorsheet","work_full_supervisorsheet"};//监理联系单
	public static final String[] PD_WORKVISA  = new String[]{"workVisa","work_visa"};
	public static final String[] PD_WORASKCHECKPRICE  = new String[]{"workaskcheckprice","work_askcheckprice"};
	public static final String[] PD_WORKCOMPLETE= new String[]{"projectCompleteAudit", "work_project_completion"};//项目竣工
	public static final String[] PD_CASEINFO= new String[]{"caseInfo", "case_info"};//案例
	public static final String[] PD_PROJECTRECORD= new String[]{"projectAudit"};//案例
	//	/**
//	 * 流程定义Map（自动初始化）
//	 */
//	private static Map<String, String> procDefMap = new HashMap<String, String>() {
//		private static final long serialVersionUID = 1L;
//		{
//			for (Field field : ActUtils.class.getFields()){
//				if(StringUtils.startsWith(field.getName(), "PD_")){
//					try{
//						String[] ss = (String[])field.get(null);
//						put(ss[0], ss[1]);
//					}catch (Exception e) {
//						logger.debug("load pd error: {}", field.getName());
//					}
//				}
//			}
//		}
//	};
//	
//	/**
//	 * 获取流程执行（办理）URL
//	 * @param procId
//	 * @return
//	 */
//	public static String getProcExeUrl(String procId) {
//		String url = procDefMap.get(StringUtils.split(procId, ":")[0]);
//		if (StringUtils.isBlank(url)){
//			return "404";
//		}
//		return url;
//	}
	
	@SuppressWarnings({ "unused" })
	public static Map<String, Object> getMobileEntity(Object entity,String spiltType){
		if(spiltType==null){
			spiltType="@";
		}
		Map<String, Object> map = Maps.newHashMap();

		List<String> field = Lists.newArrayList();
		List<String> value = Lists.newArrayList();
		List<String> chinesName =Lists.newArrayList();
		
		try{
			for (Method m : entity.getClass().getMethods()){
				if (m.getAnnotation(JsonIgnore.class) == null && m.getAnnotation(JsonBackReference.class) == null && m.getName().startsWith("get")){
					if (m.isAnnotationPresent(FieldName.class)) {
						Annotation p = m.getAnnotation(FieldName.class);
						FieldName fieldName=(FieldName) p;
						chinesName.add(fieldName.value());
					}else{
						chinesName.add("");
					}
					if (m.getName().equals("getAct")){
						Object act = m.invoke(entity, new Object[]{});
						Method actMet = act.getClass().getMethod("getTaskId");
						map.put("taskId", ObjectUtils.toString(m.invoke(act, new Object[]{}), ""));
					}else{
						field.add(StringUtils.uncapitalize(m.getName().substring(3)));
						value.add(ObjectUtils.toString(m.invoke(entity, new Object[]{}), ""));
					}
				}
			}
		}catch (Exception e) {
			e.printStackTrace();
		}
		
		map.put("beanTitles", StringUtils.join(field, spiltType));
		map.put("beanInfos", StringUtils.join(value, spiltType));
		map.put("chineseNames", StringUtils.join(chinesName, spiltType));
		
		return map;
	}
	
	/**
	 * 获取流程表单URL
	 * @param formKey
	 * @param act 表单传递参数
	 * @return
	 */
	public static String getFormUrl(String formKey, Act act){	
		StringBuilder formUrl = new StringBuilder();	
		String formServerUrl = Global.getConfig("activiti.form.server.url");
		if (StringUtils.isBlank(formServerUrl)){
			formUrl.append(Global.getAdminPath());
		}else{
			formUrl.append(formServerUrl);
		}
		
		formUrl.append(formKey).append(formUrl.indexOf("?") == -1 ? "?" : "&");
		formUrl.append("act.taskId=").append(act.getTaskId() != null ? act.getTaskId() : "");
		formUrl.append("&act.taskName=").append(act.getTaskName() != null ? Encodes.urlEncode(act.getTaskName()) : "");
		formUrl.append("&act.taskDefKey=").append(act.getTaskDefKey() != null ? act.getTaskDefKey() : "");
		formUrl.append("&act.procInsId=").append(act.getProcInsId() != null ? act.getProcInsId() : "");
		formUrl.append("&act.procDefId=").append(act.getProcDefId() != null ? act.getProcDefId() : "");
		formUrl.append("&act.status=").append(act.getStatus() != null ? act.getStatus() : "");
		formUrl.append("&id=").append(act.getBusinessId() != null ? act.getBusinessId() : "");
		
		return formUrl.toString();
	}
	
	/**
	 * 转换流程节点类型为中文说明
	 * @param type 英文名称
	 * @return 翻译后的中文名称
	 */
	public static String parseToZhType(String type) {
		Map<String, String> types = new HashMap<String, String>();
		types.put("userTask", "用户任务");
		types.put("serviceTask", "系统任务");
		types.put("startEvent", "开始节点");
		types.put("endEvent", "结束节点");
		types.put("exclusiveGateway", "条件判断节点(系统自动根据条件处理)");
		types.put("inclusiveGateway", "并行处理任务");
		types.put("callActivity", "子流程");
		return types.get(type) == null ? type : types.get(type);
	}

	public static UserEntity toActivitiUser(User user){
		if (user == null){
			return null;
		}
		UserEntity userEntity = new UserEntity();
		userEntity.setId(user.getId());
		userEntity.setFirstName(user.getName());
		userEntity.setLastName(StringUtils.EMPTY);
		userEntity.setPassword(user.getPassword());
		userEntity.setEmail(user.getEmail());
		userEntity.setRevision(1);
		return userEntity;
	}
	
	public static GroupEntity toActivitiGroup(SysRoleActivity role){
		if (role == null){
			return null;
		}
		GroupEntity groupEntity = new GroupEntity();
		groupEntity.setId(role.getEnname());
		groupEntity.setName(role.getName());
		groupEntity.setType("1");
		groupEntity.setRevision(1);
		return groupEntity;
	}
	
	public static void main(String[] args) {
		 User user = new User();
		 System.out.println(getMobileEntity(user, "@"));
	}
}
