/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.casedesigninfo.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.casedesigninfo.entity.CaseDesignInfo;
import com.jeeplus.modules.casedesigninfo.service.CaseDesignInfoService;

/**
 * 案例-设计单位Controller
 * @author fgy
 * @version 2018-01-24
 */
@Controller
@RequestMapping(value = "${adminPath}/casedesigninfo/caseDesignInfo")
public class CaseDesignInfoController extends BaseController {

	@Autowired
	private CaseDesignInfoService caseDesignInfoService;
	
	@ModelAttribute
	public CaseDesignInfo get(@RequestParam(required=false) String id) {
		CaseDesignInfo entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = caseDesignInfoService.get(id);
		}
		if (entity == null){
			entity = new CaseDesignInfo();
		}
		return entity;
	}
	
	/**
	 * 案例-设计单位列表页面
	 */
	@RequiresPermissions("casedesigninfo:caseDesignInfo:list")
	@RequestMapping(value = {"list", ""})
	public String list(CaseDesignInfo caseDesignInfo, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<CaseDesignInfo> page = caseDesignInfoService.findPage(new Page<CaseDesignInfo>(request, response), caseDesignInfo); 
		model.addAttribute("page", page);
		return "modules/casedesigninfo/caseDesignInfoList";
	}

	/**
	 * 查看，增加，编辑案例-设计单位表单页面
	 */
	@RequiresPermissions(value={"casedesigninfo:caseDesignInfo:view","casedesigninfo:caseDesignInfo:add","casedesigninfo:caseDesignInfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(CaseDesignInfo caseDesignInfo, Model model) {
		model.addAttribute("caseDesignInfo", caseDesignInfo);
		return "modules/casedesigninfo/caseDesignInfoForm";
	}

	/**
	 * 保存案例-设计单位
	 */
	@RequiresPermissions(value={"casedesigninfo:caseDesignInfo:add","casedesigninfo:caseDesignInfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(CaseDesignInfo caseDesignInfo, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, caseDesignInfo)){
			return form(caseDesignInfo, model);
		}
		if(!caseDesignInfo.getIsNewRecord()){//编辑表单保存
			CaseDesignInfo t = caseDesignInfoService.get(caseDesignInfo.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(caseDesignInfo, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			caseDesignInfoService.save(t);//保存
		}else{//新增表单保存
			caseDesignInfoService.save(caseDesignInfo);//保存
		}
		addMessage(redirectAttributes, "保存案例-设计单位成功");
		return "redirect:"+Global.getAdminPath()+"/casedesigninfo/caseDesignInfo/?repage";
	}
	
	/**
	 * 删除案例-设计单位
	 */
	@RequiresPermissions("casedesigninfo:caseDesignInfo:del")
	@RequestMapping(value = "delete")
	public String delete(CaseDesignInfo caseDesignInfo, RedirectAttributes redirectAttributes) {
		caseDesignInfoService.delete(caseDesignInfo);
		addMessage(redirectAttributes, "删除案例-设计单位成功");
		return "redirect:"+Global.getAdminPath()+"/casedesigninfo/caseDesignInfo/?repage";
	}
	
	/**
	 * 批量删除案例-设计单位
	 */
	@RequiresPermissions("casedesigninfo:caseDesignInfo:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			caseDesignInfoService.delete(caseDesignInfoService.get(id));
		}
		addMessage(redirectAttributes, "删除案例-设计单位成功");
		return "redirect:"+Global.getAdminPath()+"/casedesigninfo/caseDesignInfo/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("casedesigninfo:caseDesignInfo:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(CaseDesignInfo caseDesignInfo, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "案例-设计单位"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<CaseDesignInfo> page = caseDesignInfoService.findPage(new Page<CaseDesignInfo>(request, response, -1), caseDesignInfo);
    		new ExportExcel("案例-设计单位", CaseDesignInfo.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出案例-设计单位记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casedesigninfo/caseDesignInfo/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("casedesigninfo:caseDesignInfo:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<CaseDesignInfo> list = ei.getDataList(CaseDesignInfo.class);
			for (CaseDesignInfo caseDesignInfo : list){
				try{
					caseDesignInfoService.save(caseDesignInfo);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条案例-设计单位记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条案例-设计单位记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入案例-设计单位失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casedesigninfo/caseDesignInfo/?repage";
    }
	
	/**
	 * 下载导入案例-设计单位数据模板
	 */
	@RequiresPermissions("casedesigninfo:caseDesignInfo:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "案例-设计单位数据导入模板.xlsx";
    		List<CaseDesignInfo> list = Lists.newArrayList(); 
    		new ExportExcel("案例-设计单位数据", CaseDesignInfo.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casedesigninfo/caseDesignInfo/?repage";
    }
	
	
	

}