/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.caseexecuteinfo.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.caseexecuteinfo.entity.CaseExecuteInfo;
import com.jeeplus.modules.caseexecuteinfo.service.CaseExecuteInfoService;

/**
 * 案例-施工单位Controller
 * @author fgy
 * @version 2018-01-24
 */
@Controller
@RequestMapping(value = "${adminPath}/caseexecuteinfo/caseExecuteInfo")
public class CaseExecuteInfoController extends BaseController {

	@Autowired
	private CaseExecuteInfoService caseExecuteInfoService;
	
	@ModelAttribute
	public CaseExecuteInfo get(@RequestParam(required=false) String id) {
		CaseExecuteInfo entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = caseExecuteInfoService.get(id);
		}
		if (entity == null){
			entity = new CaseExecuteInfo();
		}
		return entity;
	}
	
	/**
	 * 案例-施工单位列表页面
	 */
	@RequiresPermissions("caseexecuteinfo:caseExecuteInfo:list")
	@RequestMapping(value = {"list", ""})
	public String list(CaseExecuteInfo caseExecuteInfo, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<CaseExecuteInfo> page = caseExecuteInfoService.findPage(new Page<CaseExecuteInfo>(request, response), caseExecuteInfo); 
		model.addAttribute("page", page);
		return "modules/caseexecuteinfo/caseExecuteInfoList";
	}

	/**
	 * 查看，增加，编辑案例-施工单位表单页面
	 */
	@RequiresPermissions(value={"caseexecuteinfo:caseExecuteInfo:view","caseexecuteinfo:caseExecuteInfo:add","caseexecuteinfo:caseExecuteInfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(CaseExecuteInfo caseExecuteInfo, Model model) {
		model.addAttribute("caseExecuteInfo", caseExecuteInfo);
		return "modules/caseexecuteinfo/caseExecuteInfoForm";
	}

	/**
	 * 保存案例-施工单位
	 */
	@RequiresPermissions(value={"caseexecuteinfo:caseExecuteInfo:add","caseexecuteinfo:caseExecuteInfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(CaseExecuteInfo caseExecuteInfo, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, caseExecuteInfo)){
			return form(caseExecuteInfo, model);
		}
		if(!caseExecuteInfo.getIsNewRecord()){//编辑表单保存
			CaseExecuteInfo t = caseExecuteInfoService.get(caseExecuteInfo.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(caseExecuteInfo, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			caseExecuteInfoService.save(t);//保存
		}else{//新增表单保存
			caseExecuteInfoService.save(caseExecuteInfo);//保存
		}
		addMessage(redirectAttributes, "保存案例-施工单位成功");
		return "redirect:"+Global.getAdminPath()+"/caseexecuteinfo/caseExecuteInfo/?repage";
	}
	
	/**
	 * 删除案例-施工单位
	 */
	@RequiresPermissions("caseexecuteinfo:caseExecuteInfo:del")
	@RequestMapping(value = "delete")
	public String delete(CaseExecuteInfo caseExecuteInfo, RedirectAttributes redirectAttributes) {
		caseExecuteInfoService.delete(caseExecuteInfo);
		addMessage(redirectAttributes, "删除案例-施工单位成功");
		return "redirect:"+Global.getAdminPath()+"/caseexecuteinfo/caseExecuteInfo/?repage";
	}
	
	/**
	 * 批量删除案例-施工单位
	 */
	@RequiresPermissions("caseexecuteinfo:caseExecuteInfo:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			caseExecuteInfoService.delete(caseExecuteInfoService.get(id));
		}
		addMessage(redirectAttributes, "删除案例-施工单位成功");
		return "redirect:"+Global.getAdminPath()+"/caseexecuteinfo/caseExecuteInfo/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("caseexecuteinfo:caseExecuteInfo:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(CaseExecuteInfo caseExecuteInfo, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "案例-施工单位"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<CaseExecuteInfo> page = caseExecuteInfoService.findPage(new Page<CaseExecuteInfo>(request, response, -1), caseExecuteInfo);
    		new ExportExcel("案例-施工单位", CaseExecuteInfo.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出案例-施工单位记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/caseexecuteinfo/caseExecuteInfo/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("caseexecuteinfo:caseExecuteInfo:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<CaseExecuteInfo> list = ei.getDataList(CaseExecuteInfo.class);
			for (CaseExecuteInfo caseExecuteInfo : list){
				try{
					caseExecuteInfoService.save(caseExecuteInfo);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条案例-施工单位记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条案例-施工单位记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入案例-施工单位失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/caseexecuteinfo/caseExecuteInfo/?repage";
    }
	
	/**
	 * 下载导入案例-施工单位数据模板
	 */
	@RequiresPermissions("caseexecuteinfo:caseExecuteInfo:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "案例-施工单位数据导入模板.xlsx";
    		List<CaseExecuteInfo> list = Lists.newArrayList(); 
    		new ExportExcel("案例-施工单位数据", CaseExecuteInfo.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/caseexecuteinfo/caseExecuteInfo/?repage";
    }
	
	
	

}