/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.casematerialsnorm.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.casematerialsnorm.entity.CaseMaterialsNorm;
import com.jeeplus.modules.casematerialsnorm.service.CaseMaterialsNormService;

/**
 * 材料消耗量指标Controller
 * @author liuw
 * @version 2018-02-07
 */
@Controller
@RequestMapping(value = "${adminPath}/casematerialsnorm/caseMaterialsNorm")
public class CaseMaterialsNormController extends BaseController {

	@Autowired
	private CaseMaterialsNormService caseMaterialsNormService;
	
	@ModelAttribute
	public CaseMaterialsNorm get(@RequestParam(required=false) String id) {
		CaseMaterialsNorm entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = caseMaterialsNormService.get(id);
		}
		if (entity == null){
			entity = new CaseMaterialsNorm();
		}
		return entity;
	}
	
	/**
	 * 材料消耗量指标列表页面
	 */
	@RequiresPermissions("casematerialsnorm:caseMaterialsNorm:list")
	@RequestMapping(value = {"list", ""})
	public String list(CaseMaterialsNorm caseMaterialsNorm, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<CaseMaterialsNorm> page = caseMaterialsNormService.findPage(new Page<CaseMaterialsNorm>(request, response), caseMaterialsNorm); 
		model.addAttribute("page", page);
		return "modules/casematerialsnorm/caseMaterialsNormList";
	}

	/**
	 * 查看，增加，编辑材料消耗量指标表单页面
	 */
	@RequiresPermissions(value={"casematerialsnorm:caseMaterialsNorm:view","casematerialsnorm:caseMaterialsNorm:add","casematerialsnorm:caseMaterialsNorm:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(CaseMaterialsNorm caseMaterialsNorm, Model model) {
		model.addAttribute("caseMaterialsNorm", caseMaterialsNorm);
		return "modules/casematerialsnorm/caseMaterialsNormForm";
	}

	/**
	 * 保存材料消耗量指标
	 */
	@RequiresPermissions(value={"casematerialsnorm:caseMaterialsNorm:add","casematerialsnorm:caseMaterialsNorm:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(CaseMaterialsNorm caseMaterialsNorm, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, caseMaterialsNorm)){
			return form(caseMaterialsNorm, model);
		}
		if(!caseMaterialsNorm.getIsNewRecord()){//编辑表单保存
			CaseMaterialsNorm t = caseMaterialsNormService.get(caseMaterialsNorm.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(caseMaterialsNorm, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			caseMaterialsNormService.save(t);//保存
		}else{//新增表单保存
			caseMaterialsNormService.save(caseMaterialsNorm);//保存
		}
		addMessage(redirectAttributes, "保存材料消耗量指标成功");
		return "redirect:"+Global.getAdminPath()+"/casematerialsnorm/caseMaterialsNorm/?repage";
	}
	
	/**
	 * 删除材料消耗量指标
	 */
	@RequiresPermissions("casematerialsnorm:caseMaterialsNorm:del")
	@RequestMapping(value = "delete")
	public String delete(CaseMaterialsNorm caseMaterialsNorm, RedirectAttributes redirectAttributes) {
		caseMaterialsNormService.delete(caseMaterialsNorm);
		addMessage(redirectAttributes, "删除材料消耗量指标成功");
		return "redirect:"+Global.getAdminPath()+"/casematerialsnorm/caseMaterialsNorm/?repage";
	}
	
	/**
	 * 批量删除材料消耗量指标
	 */
	@RequiresPermissions("casematerialsnorm:caseMaterialsNorm:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			caseMaterialsNormService.delete(caseMaterialsNormService.get(id));
		}
		addMessage(redirectAttributes, "删除材料消耗量指标成功");
		return "redirect:"+Global.getAdminPath()+"/casematerialsnorm/caseMaterialsNorm/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("casematerialsnorm:caseMaterialsNorm:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(CaseMaterialsNorm caseMaterialsNorm, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "材料消耗量指标"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<CaseMaterialsNorm> page = caseMaterialsNormService.findPage(new Page<CaseMaterialsNorm>(request, response, -1), caseMaterialsNorm);
    		new ExportExcel("材料消耗量指标", CaseMaterialsNorm.class).setDataList(page.getList()).write(response, fileName,request.getHeader("USER-AGENT")).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出材料消耗量指标记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casematerialsnorm/caseMaterialsNorm/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("casematerialsnorm:caseMaterialsNorm:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<CaseMaterialsNorm> list = ei.getDataList(CaseMaterialsNorm.class);
			for (CaseMaterialsNorm caseMaterialsNorm : list){
				try{
					caseMaterialsNormService.save(caseMaterialsNorm);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条材料消耗量指标记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条材料消耗量指标记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入材料消耗量指标失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casematerialsnorm/caseMaterialsNorm/?repage";
    }
	
	/**
	 * 下载导入材料消耗量指标数据模板
	 */
	@RequiresPermissions("casematerialsnorm:caseMaterialsNorm:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletRequest request,HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "材料消耗量指标数据导入模板.xlsx";
    		List<CaseMaterialsNorm> list = Lists.newArrayList(); 
    		new ExportExcel("材料消耗量指标数据", CaseMaterialsNorm.class, 1).setDataList(list).write(response, fileName,request.getHeader("USER-AGENT")).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casematerialsnorm/caseMaterialsNorm/?repage";
    }
	
	
	

}