/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.casemember.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.casemember.entity.CaseMember;
import com.jeeplus.modules.casemember.service.CaseMemberService;

/**
 * 会员案例Controller
 * @author lw
 * @version 2018-04-19
 */
@Controller
@RequestMapping(value = "${adminPath}/casemember/caseMember")
public class CaseMemberController extends BaseController {

	@Autowired
	private CaseMemberService caseMemberService;
	
	@ModelAttribute
	public CaseMember get(@RequestParam(required=false) String id) {
		CaseMember entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = caseMemberService.get(id);
		}
		if (entity == null){
			entity = new CaseMember();
		}
		return entity;
	}
	
	/**
	 * 会员案例列表页面
	 */
	@RequiresPermissions("casemember:caseMember:list")
	@RequestMapping(value = {"list", ""})
	public String list(CaseMember caseMember, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<CaseMember> page = caseMemberService.findPage(new Page<CaseMember>(request, response), caseMember); 
		model.addAttribute("page", page);
		return "modules/casemember/caseMemberList";
	}

	/**
	 * 查看，增加，编辑会员案例表单页面
	 */
	@RequiresPermissions(value={"casemember:caseMember:view","casemember:caseMember:add","casemember:caseMember:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(CaseMember caseMember, Model model) {
		model.addAttribute("caseMember", caseMember);
		return "modules/casemember/caseMemberForm";
	}

	/**
	 * 保存会员案例
	 */
	@RequiresPermissions(value={"casemember:caseMember:add","casemember:caseMember:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(CaseMember caseMember, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, caseMember)){
			return form(caseMember, model);
		}
		if(!caseMember.getIsNewRecord()){//编辑表单保存
			CaseMember t = caseMemberService.get(caseMember.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(caseMember, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			caseMemberService.save(t);//保存
		}else{//新增表单保存
			caseMemberService.save(caseMember);//保存
		}
		addMessage(redirectAttributes, "保存会员案例成功");
		return "redirect:"+Global.getAdminPath()+"/casemember/caseMember/?repage";
	}
	
	/**
	 * 删除会员案例
	 */
	@RequiresPermissions("casemember:caseMember:del")
	@RequestMapping(value = "delete")
	public String delete(CaseMember caseMember, RedirectAttributes redirectAttributes) {
		caseMemberService.delete(caseMember);
		addMessage(redirectAttributes, "删除会员案例成功");
		return "redirect:"+Global.getAdminPath()+"/casemember/caseMember/?repage";
	}
	
	/**
	 * 批量删除会员案例
	 */
	@RequiresPermissions("casemember:caseMember:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			caseMemberService.delete(caseMemberService.get(id));
		}
		addMessage(redirectAttributes, "删除会员案例成功");
		return "redirect:"+Global.getAdminPath()+"/casemember/caseMember/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("casemember:caseMember:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(CaseMember caseMember, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "会员案例"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<CaseMember> page = caseMemberService.findPage(new Page<CaseMember>(request, response, -1), caseMember);
    		new ExportExcel("会员案例", CaseMember.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出会员案例记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casemember/caseMember/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("casemember:caseMember:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<CaseMember> list = ei.getDataList(CaseMember.class);
			for (CaseMember caseMember : list){
				try{
					caseMemberService.save(caseMember);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条会员案例记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条会员案例记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入会员案例失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casemember/caseMember/?repage";
    }
	
	/**
	 * 下载导入会员案例数据模板
	 */
	@RequiresPermissions("casemember:caseMember:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "会员案例数据导入模板.xlsx";
    		List<CaseMember> list = Lists.newArrayList(); 
    		new ExportExcel("会员案例数据", CaseMember.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/casemember/caseMember/?repage";
    }
	
	
	

}