/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.caseprojectfeature.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.caseprojectfeature.entity.CaseProjectFeature;
import com.jeeplus.modules.caseprojectfeature.service.CaseProjectFeatureService;

/**
 * 案例-主要项目特征Controller
 * @author fgy
 * @version 2018-01-24
 */
@Controller
@RequestMapping(value = "${adminPath}/caseprojectfeature/caseProjectFeature")
public class CaseProjectFeatureController extends BaseController {

	@Autowired
	private CaseProjectFeatureService caseProjectFeatureService;
	
	@ModelAttribute
	public CaseProjectFeature get(@RequestParam(required=false) String id) {
		CaseProjectFeature entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = caseProjectFeatureService.get(id);
		}
		if (entity == null){
			entity = new CaseProjectFeature();
		}
		return entity;
	}
	
	/**
	 * 案例-主要项目特征列表页面
	 */
	@RequiresPermissions("caseprojectfeature:caseProjectFeature:list")
	@RequestMapping(value = {"list", ""})
	public String list(CaseProjectFeature caseProjectFeature, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<CaseProjectFeature> page = caseProjectFeatureService.findPage(new Page<CaseProjectFeature>(request, response), caseProjectFeature); 
		model.addAttribute("page", page);
		return "modules/caseprojectfeature/caseProjectFeatureList";
	}

	/**
	 * 查看，增加，编辑案例-主要项目特征表单页面
	 */
	@RequiresPermissions(value={"caseprojectfeature:caseProjectFeature:view","caseprojectfeature:caseProjectFeature:add","caseprojectfeature:caseProjectFeature:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(CaseProjectFeature caseProjectFeature, Model model) {
		model.addAttribute("caseProjectFeature", caseProjectFeature);
		return "modules/caseprojectfeature/caseProjectFeatureForm";
	}

	/**
	 * 保存案例-主要项目特征
	 */
	@RequiresPermissions(value={"caseprojectfeature:caseProjectFeature:add","caseprojectfeature:caseProjectFeature:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(CaseProjectFeature caseProjectFeature, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, caseProjectFeature)){
			return form(caseProjectFeature, model);
		}
		if(!caseProjectFeature.getIsNewRecord()){//编辑表单保存
			CaseProjectFeature t = caseProjectFeatureService.get(caseProjectFeature.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(caseProjectFeature, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			caseProjectFeatureService.save(t);//保存
		}else{//新增表单保存
			caseProjectFeatureService.save(caseProjectFeature);//保存
		}
		addMessage(redirectAttributes, "保存案例-主要项目特征成功");
		return "redirect:"+Global.getAdminPath()+"/caseprojectfeature/caseProjectFeature/?repage";
	}
	
	/**
	 * 删除案例-主要项目特征
	 */
	@RequiresPermissions("caseprojectfeature:caseProjectFeature:del")
	@RequestMapping(value = "delete")
	public String delete(CaseProjectFeature caseProjectFeature, RedirectAttributes redirectAttributes) {
		caseProjectFeatureService.delete(caseProjectFeature);
		addMessage(redirectAttributes, "删除案例-主要项目特征成功");
		return "redirect:"+Global.getAdminPath()+"/caseprojectfeature/caseProjectFeature/?repage";
	}
	
	/**
	 * 批量删除案例-主要项目特征
	 */
	@RequiresPermissions("caseprojectfeature:caseProjectFeature:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			caseProjectFeatureService.delete(caseProjectFeatureService.get(id));
		}
		addMessage(redirectAttributes, "删除案例-主要项目特征成功");
		return "redirect:"+Global.getAdminPath()+"/caseprojectfeature/caseProjectFeature/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("caseprojectfeature:caseProjectFeature:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(CaseProjectFeature caseProjectFeature, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "案例-主要项目特征"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<CaseProjectFeature> page = caseProjectFeatureService.findPage(new Page<CaseProjectFeature>(request, response, -1), caseProjectFeature);
    		new ExportExcel("案例-主要项目特征", CaseProjectFeature.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出案例-主要项目特征记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/caseprojectfeature/caseProjectFeature/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("caseprojectfeature:caseProjectFeature:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<CaseProjectFeature> list = ei.getDataList(CaseProjectFeature.class);
			for (CaseProjectFeature caseProjectFeature : list){
				try{
					caseProjectFeatureService.save(caseProjectFeature);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条案例-主要项目特征记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条案例-主要项目特征记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入案例-主要项目特征失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/caseprojectfeature/caseProjectFeature/?repage";
    }
	
	/**
	 * 下载导入案例-主要项目特征数据模板
	 */
	@RequiresPermissions("caseprojectfeature:caseProjectFeature:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "案例-主要项目特征数据导入模板.xlsx";
    		List<CaseProjectFeature> list = Lists.newArrayList(); 
    		new ExportExcel("案例-主要项目特征数据", CaseProjectFeature.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/caseprojectfeature/caseProjectFeature/?repage";
    }
	
	
	

}