/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.examplesingle.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.examplesingle.entity.ExampleSingle;
import com.jeeplus.modules.examplesingle.service.ExampleSingleService;

/**
 * 单项工程造价指标Controller
 * @author ssrh
 * @version 2018-08-29
 */
@Controller
@RequestMapping(value = "${adminPath}/examplesingle/exampleSingle")
public class ExampleSingleController extends BaseController {

	@Autowired
	private ExampleSingleService exampleSingleService;
	
	@ModelAttribute
	public ExampleSingle get(@RequestParam(required=false) String id) {
		ExampleSingle entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = exampleSingleService.get(id);
		}
		if (entity == null){
			entity = new ExampleSingle();
		}
		return entity;
	}
	
	/**
	 * 单项工程造价指标列表页面
	 */
	@RequiresPermissions("examplesingle:exampleSingle:list")
	@RequestMapping(value = {"list", ""})
	public String list(ExampleSingle exampleSingle, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<ExampleSingle> page = exampleSingleService.findPage(new Page<ExampleSingle>(request, response), exampleSingle); 
		model.addAttribute("page", page);
		return "modules/examplesingle/exampleSingleList";
	}

	/**
	 * 查看，增加，编辑单项工程造价指标表单页面
	 */
	@RequiresPermissions(value={"examplesingle:exampleSingle:view","examplesingle:exampleSingle:add","examplesingle:exampleSingle:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(ExampleSingle exampleSingle, Model model) {
		model.addAttribute("exampleSingle", exampleSingle);
		return "modules/examplesingle/exampleSingleForm";
	}

	/**
	 * 保存单项工程造价指标
	 */
	@RequiresPermissions(value={"examplesingle:exampleSingle:add","examplesingle:exampleSingle:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(ExampleSingle exampleSingle, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, exampleSingle)){
			return form(exampleSingle, model);
		}
		if(!exampleSingle.getIsNewRecord()){//编辑表单保存
			ExampleSingle t = exampleSingleService.get(exampleSingle.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(exampleSingle, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			exampleSingleService.save(t);//保存
		}else{//新增表单保存
			exampleSingleService.save(exampleSingle);//保存
		}
		addMessage(redirectAttributes, "保存单项工程造价指标成功");
		return "redirect:"+Global.getAdminPath()+"/examplesingle/exampleSingle/?repage";
	}
	
	/**
	 * 删除单项工程造价指标
	 */
	@RequiresPermissions("examplesingle:exampleSingle:del")
	@RequestMapping(value = "delete")
	public String delete(ExampleSingle exampleSingle, RedirectAttributes redirectAttributes) {
		exampleSingleService.delete(exampleSingle);
		addMessage(redirectAttributes, "删除单项工程造价指标成功");
		return "redirect:"+Global.getAdminPath()+"/examplesingle/exampleSingle/?repage";
	}
	
	/**
	 * 批量删除单项工程造价指标
	 */
	@RequiresPermissions("examplesingle:exampleSingle:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			exampleSingleService.delete(exampleSingleService.get(id));
		}
		addMessage(redirectAttributes, "删除单项工程造价指标成功");
		return "redirect:"+Global.getAdminPath()+"/examplesingle/exampleSingle/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("examplesingle:exampleSingle:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(ExampleSingle exampleSingle, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "单项工程造价指标"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<ExampleSingle> page = exampleSingleService.findPage(new Page<ExampleSingle>(request, response, -1), exampleSingle);
    		new ExportExcel("单项工程造价指标", ExampleSingle.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出单项工程造价指标记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/examplesingle/exampleSingle/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("examplesingle:exampleSingle:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<ExampleSingle> list = ei.getDataList(ExampleSingle.class);
			for (ExampleSingle exampleSingle : list){
				try{
					exampleSingleService.save(exampleSingle);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条单项工程造价指标记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条单项工程造价指标记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入单项工程造价指标失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/examplesingle/exampleSingle/?repage";
    }
	
	/**
	 * 下载导入单项工程造价指标数据模板
	 */
	@RequiresPermissions("examplesingle:exampleSingle:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "单项工程造价指标数据导入模板.xlsx";
    		List<ExampleSingle> list = Lists.newArrayList(); 
    		new ExportExcel("单项工程造价指标数据", ExampleSingle.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/examplesingle/exampleSingle/?repage";
    }
	
	
	

}