/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.leaveapply.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.leaveapply.entity.LeaveDetail;
import com.jeeplus.modules.leaveapply.service.LeaveDetailService;

/**
 * 请假申请Controller
 * @author ssrh
 * @version 2018-09-10
 */
@Controller
@RequestMapping(value = "${adminPath}/leaveapply/leaveDetail")
public class LeaveDetailController extends BaseController {

	@Autowired
	private LeaveDetailService leaveDetailService;
	
	@ModelAttribute
	public LeaveDetail get(@RequestParam(required=false) String id) {
		LeaveDetail entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = leaveDetailService.get(id);
		}
		if (entity == null){
			entity = new LeaveDetail();
		}
		return entity;
	}
	
	/**
	 * 请假申请列表页面
	 */
	@RequiresPermissions("leaveapply:leaveDetail:list")
	@RequestMapping(value = {"list", ""})
	public String list(LeaveDetail leaveDetail, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<LeaveDetail> page = leaveDetailService.findPage(new Page<LeaveDetail>(request, response), leaveDetail); 
		model.addAttribute("page", page);
		return "modules/leaveapply/leaveDetailList";
	}

	/**
	 * 查看，增加，编辑请假申请表单页面
	 */
	@RequiresPermissions(value={"leaveapply:leaveDetail:view","leaveapply:leaveDetail:add","leaveapply:leaveDetail:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(LeaveDetail leaveDetail, Model model) {
		model.addAttribute("leaveDetail", leaveDetail);
		return "modules/leaveapply/leaveDetailForm";
	}

	/**
	 * 保存请假申请
	 */
	@RequiresPermissions(value={"leaveapply:leaveDetail:add","leaveapply:leaveDetail:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(LeaveDetail leaveDetail, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, leaveDetail)){
			return form(leaveDetail, model);
		}
		if(!leaveDetail.getIsNewRecord()){//编辑表单保存
			LeaveDetail t = leaveDetailService.get(leaveDetail.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(leaveDetail, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			leaveDetailService.save(t);//保存
		}else{//新增表单保存
			leaveDetailService.save(leaveDetail);//保存
		}
		addMessage(redirectAttributes, "保存请假申请成功");
		return "redirect:"+Global.getAdminPath()+"/leaveapply/leaveDetail/?repage";
	}
	
	/**
	 * 删除请假申请
	 */
	@RequiresPermissions("leaveapply:leaveDetail:del")
	@RequestMapping(value = "delete")
	public String delete(LeaveDetail leaveDetail, RedirectAttributes redirectAttributes) {
		leaveDetailService.delete(leaveDetail);
		addMessage(redirectAttributes, "删除请假申请成功");
		return "redirect:"+Global.getAdminPath()+"/leaveapply/leaveDetail/?repage";
	}
	
	/**
	 * 批量删除请假申请
	 */
	@RequiresPermissions("leaveapply:leaveDetail:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			leaveDetailService.delete(leaveDetailService.get(id));
		}
		addMessage(redirectAttributes, "删除请假申请成功");
		return "redirect:"+Global.getAdminPath()+"/leaveapply/leaveDetail/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("leaveapply:leaveDetail:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(LeaveDetail leaveDetail, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "请假申请"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<LeaveDetail> page = leaveDetailService.findPage(new Page<LeaveDetail>(request, response, -1), leaveDetail);
    		new ExportExcel("请假申请", LeaveDetail.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出请假申请记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/leaveapply/leaveDetail/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("leaveapply:leaveDetail:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<LeaveDetail> list = ei.getDataList(LeaveDetail.class);
			for (LeaveDetail leaveDetail : list){
				try{
					leaveDetailService.save(leaveDetail);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条请假申请记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条请假申请记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入请假申请失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/leaveapply/leaveDetail/?repage";
    }
	
	/**
	 * 下载导入请假申请数据模板
	 */
	@RequiresPermissions("leaveapply:leaveDetail:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "请假申请数据导入模板.xlsx";
    		List<LeaveDetail> list = Lists.newArrayList(); 
    		new ExportExcel("请假申请数据", LeaveDetail.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/leaveapply/leaveDetail/?repage";
    }
	
	
	

}