/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.project.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.project.entity.ProjectEvaluationRecord;
import com.jeeplus.modules.project.service.ProjectEvaluationRecordService;

/**
 * 招标代理评价记录表Controller
 * @author liuw
 * @version 2017-10-12
 */
@Controller
@RequestMapping(value = "${adminPath}/project/projectEvaluationRecord")
public class ProjectEvaluationRecordController extends BaseController {

	@Autowired
	private ProjectEvaluationRecordService projectEvaluationRecordService;
	
	@ModelAttribute
	public ProjectEvaluationRecord get(@RequestParam(required=false) String id) {
		ProjectEvaluationRecord entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = projectEvaluationRecordService.get(id);
		}
		if (entity == null){
			entity = new ProjectEvaluationRecord();
		}
		return entity;
	}
	
	/**
	 * 招标代理评价记录表列表页面
	 */
	@RequiresPermissions("project:projectEvaluationRecord:list")
	@RequestMapping(value = {"list", ""})
	public String list(ProjectEvaluationRecord projectEvaluationRecord, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<ProjectEvaluationRecord> page = projectEvaluationRecordService.findPage(new Page<ProjectEvaluationRecord>(request, response), projectEvaluationRecord); 
		model.addAttribute("page", page);
		return "modules/project/projectEvaluationRecordList";
	}

	/**
	 * 查看，增加，编辑招标代理评价记录表表单页面
	 */
	@RequiresPermissions(value={"project:projectEvaluationRecord:view","project:projectEvaluationRecord:add","project:projectEvaluationRecord:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(ProjectEvaluationRecord projectEvaluationRecord, Model model) {
		model.addAttribute("projectEvaluationRecord", projectEvaluationRecord);
		return "modules/project/projectEvaluationRecordForm";
	}

	/**
	 * 保存招标代理评价记录表
	 */
	@RequiresPermissions(value={"project:projectEvaluationRecord:add","project:projectEvaluationRecord:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(ProjectEvaluationRecord projectEvaluationRecord, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, projectEvaluationRecord)){
			return form(projectEvaluationRecord, model);
		}
		if(!projectEvaluationRecord.getIsNewRecord()){//编辑表单保存
			ProjectEvaluationRecord t = projectEvaluationRecordService.get(projectEvaluationRecord.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(projectEvaluationRecord, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			projectEvaluationRecordService.save(t);//保存
		}else{//新增表单保存
			projectEvaluationRecordService.save(projectEvaluationRecord);//保存
		}
		addMessage(redirectAttributes, "保存招标代理评价记录表成功");
		return "redirect:"+Global.getAdminPath()+"/project/projectEvaluationRecord/?repage";
	}
	
	/**
	 * 删除招标代理评价记录表
	 */
	@RequiresPermissions("project:projectEvaluationRecord:del")
	@RequestMapping(value = "delete")
	public String delete(ProjectEvaluationRecord projectEvaluationRecord, RedirectAttributes redirectAttributes) {
		projectEvaluationRecordService.delete(projectEvaluationRecord);
		addMessage(redirectAttributes, "删除招标代理评价记录表成功");
		return "redirect:"+Global.getAdminPath()+"/project/projectEvaluationRecord/?repage";
	}
	
	/**
	 * 批量删除招标代理评价记录表
	 */
	@RequiresPermissions("project:projectEvaluationRecord:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			projectEvaluationRecordService.delete(projectEvaluationRecordService.get(id));
		}
		addMessage(redirectAttributes, "删除招标代理评价记录表成功");
		return "redirect:"+Global.getAdminPath()+"/project/projectEvaluationRecord/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("project:projectEvaluationRecord:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(ProjectEvaluationRecord projectEvaluationRecord, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "招标代理评价记录表"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<ProjectEvaluationRecord> page = projectEvaluationRecordService.findPage(new Page<ProjectEvaluationRecord>(request, response, -1), projectEvaluationRecord);
    		new ExportExcel("招标代理评价记录表", ProjectEvaluationRecord.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出招标代理评价记录表记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/project/projectEvaluationRecord/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("project:projectEvaluationRecord:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<ProjectEvaluationRecord> list = ei.getDataList(ProjectEvaluationRecord.class);
			for (ProjectEvaluationRecord projectEvaluationRecord : list){
				try{
					projectEvaluationRecordService.save(projectEvaluationRecord);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条招标代理评价记录表记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条招标代理评价记录表记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入招标代理评价记录表失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/project/projectEvaluationRecord/?repage";
    }
	
	/**
	 * 下载导入招标代理评价记录表数据模板
	 */
	@RequiresPermissions("project:projectEvaluationRecord:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "招标代理评价记录表数据导入模板.xlsx";
    		List<ProjectEvaluationRecord> list = Lists.newArrayList(); 
    		new ExportExcel("招标代理评价记录表数据", ProjectEvaluationRecord.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/project/projectEvaluationRecord/?repage";
    }
	
	
	

}