/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sysnumberinfo.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.sys.entity.User;
import com.jeeplus.modules.sysnumberinfo.entity.SysNumberInfo;
import com.jeeplus.modules.sysnumberinfo.service.SysNumberInfoService;

/**
 * 编码制定Controller
 * @author 丁旭
 * @version 2017-03-22
 */
@Controller
@RequestMapping(value = "${adminPath}/sysnumberinfo/sysNumberInfo")
public class SysNumberInfoController extends BaseController {

	@Autowired
	private SysNumberInfoService sysNumberInfoService;
	

	
	@ModelAttribute
	public SysNumberInfo get(@RequestParam(required=false) String id) {
		SysNumberInfo entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = sysNumberInfoService.get(id);
		}
		if (entity == null){
			entity = new SysNumberInfo();
		}
		return entity;
	}
	
	
	
	@RequiresPermissions("sysnumberinfo:sysNumberInfo:index")
	@RequestMapping(value = {"index"})
	public String index(User user, Model model) {
		return "modules/sysnumberinfo/sysNumberInfoIndex";
	}
	/**
	 * 编码制定列表页面
	 */
	@RequiresPermissions("sysnumberinfo:sysNumberInfo:index")
	@RequestMapping(value = {"list", ""})
	public String list(SysNumberInfo sysNumberInfo, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<SysNumberInfo> page = sysNumberInfoService.findPage(new Page<SysNumberInfo>(request, response), sysNumberInfo); 
		model.addAttribute("page", page);
		return "modules/sysnumberinfo/sysNumberInfoList";
	}

	/**
	 * 查看，增加，编辑编码制定表单页面
	 */
	@RequiresPermissions(value={"sysnumberinfo:sysNumberInfo:view","sysnumberinfo:sysNumberInfo:add","sysnumberinfo:sysNumberInfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(SysNumberInfo sysNumberInfo, Model model) {
		model.addAttribute("sysNumberInfo", sysNumberInfo);
		return "modules/sysnumberinfo/sysNumberInfoForm";
	}

	/**
	 * 保存编码制定
	 */
	@RequiresPermissions(value={"sysnumberinfo:sysNumberInfo:add","sysnumberinfo:sysNumberInfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(SysNumberInfo sysNumberInfo, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, sysNumberInfo)){
			return form(sysNumberInfo, model);
		}
		if(!sysNumberInfo.getIsNewRecord()){//编辑表单保存
			SysNumberInfo t = sysNumberInfoService.get(sysNumberInfo.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(sysNumberInfo, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			sysNumberInfoService.save(t);//保存
		}else{//新增表单保存
			sysNumberInfoService.save(sysNumberInfo);//保存
		}
		addMessage(redirectAttributes, "保存编码制定成功");
		return "redirect:"+Global.getAdminPath()+"/sysnumberinfo/sysNumberInfo/index?repage";
	}
	
	/**
	 * 删除编码制定
	 */
	@RequiresPermissions("sysnumberinfo:sysNumberInfo:del")
	@RequestMapping(value = "delete")
	public String delete(SysNumberInfo sysNumberInfo, RedirectAttributes redirectAttributes) {
		sysNumberInfoService.delete(sysNumberInfo);
		addMessage(redirectAttributes, "删除编码制定成功");
		return "redirect:"+Global.getAdminPath()+"/sysnumberinfo/sysNumberInfo/index?repage";
	}
	
	/**
	 * 批量删除编码制定
	 */
	@RequiresPermissions("sysnumberinfo:sysNumberInfo:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			sysNumberInfoService.delete(sysNumberInfoService.get(id));
		}
		addMessage(redirectAttributes, "删除编码制定成功");
		return "redirect:"+Global.getAdminPath()+"/sysnumberinfo/sysNumberInfo/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("sysnumberinfo:sysNumberInfo:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(SysNumberInfo sysNumberInfo, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "编码制定"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<SysNumberInfo> page = sysNumberInfoService.findPage(new Page<SysNumberInfo>(request, response, -1), sysNumberInfo);
    		new ExportExcel("编码制定", SysNumberInfo.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出编码制定记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysnumberinfo/sysNumberInfo/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("sysnumberinfo:sysNumberInfo:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<SysNumberInfo> list = ei.getDataList(SysNumberInfo.class);
			for (SysNumberInfo sysNumberInfo : list){
				try{
					sysNumberInfoService.save(sysNumberInfo);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条编码制定记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条编码制定记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入编码制定失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysnumberinfo/sysNumberInfo/?repage";
    }
	
	/**
	 * 下载导入编码制定数据模板
	 */
	@RequiresPermissions("sysnumberinfo:sysNumberInfo:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "编码制定数据导入模板.xlsx";
    		List<SysNumberInfo> list = Lists.newArrayList(); 
    		new ExportExcel("编码制定数据", SysNumberInfo.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysnumberinfo/sysNumberInfo/?repage";
    }
	
	
	

}