/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workattendance.web;

import java.text.ParseException;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.modules.workclientinfo.entity.WorkClientAttachment;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workattendance.entity.WorkAttendanceInfo;
import com.jeeplus.modules.workattendance.service.WorkAttendanceInfoService;

/**
 * 考勤详情Controller
 * @author 杨帆
 * @version 2018-09-10
 */
@Controller
@RequestMapping(value = "${adminPath}/workattendance/workAttendanceInfo")
public class WorkAttendanceInfoController extends BaseController {

	@Autowired
	private WorkAttendanceInfoService workAttendanceInfoService;
	
	@ModelAttribute
	public WorkAttendanceInfo get(@RequestParam(required=false) String id) {
		WorkAttendanceInfo entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workAttendanceInfoService.get(id);
		}
		if (entity == null){
			entity = new WorkAttendanceInfo();
		}
		return entity;
	}
	
	/**
	 * 考勤详情列表页面
	 */
	@RequiresPermissions("workattendance:workAttendanceInfo:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkAttendanceInfo workAttendanceInfo, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkAttendanceInfo> page = workAttendanceInfoService.findPage(new Page<WorkAttendanceInfo>(request, response), workAttendanceInfo); 
		model.addAttribute("page", page);
		return "modules/workattendance/workAttendanceInfoList";
	}

	/**
	 * 查看，增加，编辑考勤详情表单页面
	 */
	//@RequiresPermissions(value={"workattendance:workAttendanceInfo:view","workattendance:workAttendanceInfo:add","workattendance:workAttendanceInfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkAttendanceInfo workAttendanceInfo, Model model) {
		model.addAttribute("workAttendanceInfo", workAttendanceInfo);
		if (StringUtils.isNotBlank(workAttendanceInfo.getHome())){
			return "modules/workattendance/workAttendanceInfoEdit";
		}else {
			return "modules/workattendance/workAttendanceInfoForm";
		}
	}
	/**
	 * 查看，增加，编辑考勤详情表单页面
	 */
	@RequestMapping(value = "view")
	public String view(WorkAttendanceInfo workAttendanceInfo, Model model) {
		List<WorkAttendanceInfo> attendanceInfos = workAttendanceInfoService.findLists(workAttendanceInfo);
		model.addAttribute("workAttendanceInfos", attendanceInfos);
		model.addAttribute("home", workAttendanceInfo.getHome());
		return "modules/workattendance/workAttendanceInfoView";
	}

	/**
	 * 保存考勤规则
	 */
	@RequestMapping(value = "saveAudit")
	public String saveAudit(WorkAttendanceInfo workAttendanceInfo, Model model, HttpServletRequest request, RedirectAttributes redirectAttributes) throws Exception{
		workAttendanceInfoService.saveAudit(workAttendanceInfo.getId(),workAttendanceInfo.getIds(),workAttendanceInfo.getFlag());//保存
		addMessage(redirectAttributes, "审批成功");
		if (StringUtils.isNotBlank(workAttendanceInfo.getHome()) && "home".equals(workAttendanceInfo.getHome())){
			return "redirect:" + Global.getAdminPath() + "/home/?repage";
		}else {
			return "redirect:"+Global.getAdminPath()+"/workattendance/workAttendanceRule/?repage";
		}
	}

	/**
	 * 保存考勤详情
	 */
	//@RequiresPermissions(value={"workattendance:workAttendanceInfo:add","workattendance:workAttendanceInfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkAttendanceInfo workAttendanceInfo, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workAttendanceInfo)){
			return form(workAttendanceInfo, model);
		}
		if(!workAttendanceInfo.getIsNewRecord()){//编辑表单保存
			WorkAttendanceInfo t = workAttendanceInfoService.get(workAttendanceInfo.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workAttendanceInfo, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workAttendanceInfoService.save(t);//保存
		}else{//新增表单保存
			workAttendanceInfoService.save(workAttendanceInfo);//保存
		}
		addMessage(redirectAttributes, "保存考勤详情成功");
		return "redirect:"+Global.getAdminPath()+"/workattendance/workAttendanceInfo/?repage";
	}
	
	/**
	 * 删除考勤详情
	 */
	@RequiresPermissions("workattendance:workAttendanceInfo:del")
	@RequestMapping(value = "delete")
	public String delete(WorkAttendanceInfo workAttendanceInfo, RedirectAttributes redirectAttributes) {
		workAttendanceInfoService.delete(workAttendanceInfo);
		addMessage(redirectAttributes, "删除考勤详情成功");
		return "redirect:"+Global.getAdminPath()+"/workattendance/workAttendanceInfo/?repage";
	}
	
	/**
	 * 批量删除考勤详情
	 */
	@RequiresPermissions("workattendance:workAttendanceInfo:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workAttendanceInfoService.delete(workAttendanceInfoService.get(id));
		}
		addMessage(redirectAttributes, "删除考勤详情成功");
		return "redirect:"+Global.getAdminPath()+"/workattendance/workAttendanceInfo/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workattendance:workAttendanceInfo:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkAttendanceInfo workAttendanceInfo, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "考勤详情"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkAttendanceInfo> page = workAttendanceInfoService.findPage(new Page<WorkAttendanceInfo>(request, response, -1), workAttendanceInfo);
    		new ExportExcel("考勤详情", WorkAttendanceInfo.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出考勤详情记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workattendance/workAttendanceInfo/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workattendance:workAttendanceInfo:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkAttendanceInfo> list = ei.getDataList(WorkAttendanceInfo.class);
			for (WorkAttendanceInfo workAttendanceInfo : list){
				try{
					workAttendanceInfoService.save(workAttendanceInfo);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条考勤详情记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条考勤详情记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入考勤详情失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workattendance/workAttendanceInfo/?repage";
    }
	
	/**
	 * 下载导入考勤详情数据模板
	 */
	@RequiresPermissions("workattendance:workAttendanceInfo:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "考勤详情数据导入模板.xlsx";
    		List<WorkAttendanceInfo> list = Lists.newArrayList(); 
    		new ExportExcel("考勤详情数据", WorkAttendanceInfo.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workattendance/workAttendanceInfo/?repage";
    }
	
	
	

}