/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workcontent.web;

import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.modules.sys.utils.UserUtils;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workcontent.entity.WorkContentClaim;
import com.jeeplus.modules.workcontent.service.WorkContentClaimService;

/**
 * 索赔管理Controller
 * @author ssrh
 * @version 2018-06-12
 */
@Controller
@RequestMapping(value = "${adminPath}/workcontent/workContentClaim")
public class WorkContentClaimController extends BaseController {

	@Autowired
	private WorkContentClaimService workContentClaimService;
	
	@ModelAttribute
	public WorkContentClaim get(@RequestParam(required=false) String id) {
		WorkContentClaim entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workContentClaimService.get(id);
		}
		if (entity == null){
			entity = new WorkContentClaim();
		}
		return entity;
	}
	
	/**
	 * 索赔管理列表页面
	 */
	@RequiresPermissions("workcontent:workContentClaim:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkContentClaim workContentClaim, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkContentClaim> page = workContentClaimService.findPage(new Page<WorkContentClaim>(request, response), workContentClaim); 
		model.addAttribute("page", page);
		return "modules/workcontent/workContentClaimList";
	}

	/**
	 * 查看，增加，编辑索赔管理表单页面
	 */
	@RequestMapping(value = "form")
	public String form(WorkContentClaim workContentClaim,String view, Model model) {
        WorkContentClaim oldRecord = null;
        if(StringUtils.isNotBlank(workContentClaim.getContentId())){
            List<WorkContentClaim> list = workContentClaimService.findList(workContentClaim);
            if(list!=null&&!list.isEmpty()){
                oldRecord = list.get(0);
            }
        }
        if(oldRecord!=null){
            model.addAttribute("workContentClaim", oldRecord);
        }else {
            workContentClaim.setCreateBy(UserUtils.getUser());
            workContentClaim.setCreateDate(new Date());
            model.addAttribute("workContentClaim", workContentClaim);
        }
        String viewPath = "modules/workcontent/workContentClaimForm";
        if("view".equals(view)){
            viewPath+="View";
        }
        return viewPath;
	}

	/**
	 * 保存索赔管理
	 */
	@RequiresPermissions(value={"workcontent:workContentClaim:add","workcontent:workContentClaim:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkContentClaim workContentClaim, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workContentClaim)){
			return form(workContentClaim, null,model);
		}
		if(!workContentClaim.getIsNewRecord()){//编辑表单保存
			WorkContentClaim t = workContentClaimService.get(workContentClaim.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workContentClaim, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workContentClaimService.save(t);//保存
		}else{//新增表单保存
			workContentClaimService.save(workContentClaim);//保存
		}
		addMessage(redirectAttributes, "保存索赔管理成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentClaim/?repage";
	}
	
	/**
	 * 删除索赔管理
	 */
	@RequiresPermissions("workcontent:workContentClaim:del")
	@RequestMapping(value = "delete")
	public String delete(WorkContentClaim workContentClaim, RedirectAttributes redirectAttributes) {
		workContentClaimService.delete(workContentClaim);
		addMessage(redirectAttributes, "删除索赔管理成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentClaim/?repage";
	}
	
	/**
	 * 批量删除索赔管理
	 */
	@RequiresPermissions("workcontent:workContentClaim:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workContentClaimService.delete(workContentClaimService.get(id));
		}
		addMessage(redirectAttributes, "删除索赔管理成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentClaim/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workcontent:workContentClaim:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkContentClaim workContentClaim, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "索赔管理"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkContentClaim> page = workContentClaimService.findPage(new Page<WorkContentClaim>(request, response, -1), workContentClaim);
    		new ExportExcel("索赔管理", WorkContentClaim.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出索赔管理记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentClaim/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workcontent:workContentClaim:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkContentClaim> list = ei.getDataList(WorkContentClaim.class);
			for (WorkContentClaim workContentClaim : list){
				try{
					workContentClaimService.save(workContentClaim);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条索赔管理记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条索赔管理记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入索赔管理失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentClaim/?repage";
    }
	
	/**
	 * 下载导入索赔管理数据模板
	 */
	@RequiresPermissions("workcontent:workContentClaim:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "索赔管理数据导入模板.xlsx";
    		List<WorkContentClaim> list = Lists.newArrayList(); 
    		new ExportExcel("索赔管理数据", WorkContentClaim.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentClaim/?repage";
    }
	
	
	

}