/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workmeeting.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workmeeting.entity.Workmeetinginfo;
import com.jeeplus.modules.workmeeting.service.WorkmeetinginfoService;

/**
 * 会议信息Controller
 * @author 杨帆
 * @version 2018-03-05
 */
@Controller
@RequestMapping(value = "${adminPath}/workmeeting/workmeetinginfo")
public class WorkmeetinginfoController extends BaseController {

	@Autowired
	private WorkmeetinginfoService workmeetinginfoService;
	
	@ModelAttribute
	public Workmeetinginfo get(@RequestParam(required=false) String id) {
		Workmeetinginfo entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workmeetinginfoService.get(id);
		}
		if (entity == null){
			entity = new Workmeetinginfo();
		}
		return entity;
	}
	
	/**
	 * workmeeting列表页面
	 */
	@RequiresPermissions("workmeeting:workmeetinginfo:list")
	@RequestMapping(value = {"list", ""})
	public String list(Workmeetinginfo workmeetinginfo, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<Workmeetinginfo> page = workmeetinginfoService.findPage(new Page<Workmeetinginfo>(request, response), workmeetinginfo); 
		model.addAttribute("page", page);
		return "modules/workmeeting/workmeetinginfoList";
	}

	/**
	 * 查看，增加，编辑workmeeting表单页面
	 */
	@RequiresPermissions(value={"workmeeting:workmeetinginfo:view","workmeeting:workmeetinginfo:add","workmeeting:workmeetinginfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(Workmeetinginfo workmeetinginfo, Model model) {
		model.addAttribute("workmeetinginfo", workmeetinginfo);
		return "modules/workmeeting/workmeetinginfoForm";
	}

	/**
	 * 保存workmeeting
	 */
	@RequiresPermissions(value={"workmeeting:workmeetinginfo:add","workmeeting:workmeetinginfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(Workmeetinginfo workmeetinginfo, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workmeetinginfo)){
			return form(workmeetinginfo, model);
		}
		if(!workmeetinginfo.getIsNewRecord()){//编辑表单保存
			Workmeetinginfo t = workmeetinginfoService.get(workmeetinginfo.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workmeetinginfo, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workmeetinginfoService.save(t);//保存
		}else{//新增表单保存
			workmeetinginfoService.save(workmeetinginfo);//保存
		}
		addMessage(redirectAttributes, "保存workmeeting成功");
		return "redirect:"+Global.getAdminPath()+"/workmeeting/workmeetinginfo/?repage";
	}
	
	/**
	 * 删除workmeeting
	 */
	@RequiresPermissions("workmeeting:workmeetinginfo:del")
	@RequestMapping(value = "delete")
	public String delete(Workmeetinginfo workmeetinginfo, RedirectAttributes redirectAttributes) {
		workmeetinginfoService.delete(workmeetinginfo);
		addMessage(redirectAttributes, "删除workmeeting成功");
		return "redirect:"+Global.getAdminPath()+"/workmeeting/workmeetinginfo/?repage";
	}
	
	/**
	 * 批量删除workmeeting
	 */
	@RequiresPermissions("workmeeting:workmeetinginfo:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workmeetinginfoService.delete(workmeetinginfoService.get(id));
		}
		addMessage(redirectAttributes, "删除workmeeting成功");
		return "redirect:"+Global.getAdminPath()+"/workmeeting/workmeetinginfo/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workmeeting:workmeetinginfo:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(Workmeetinginfo workmeetinginfo, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "workmeeting"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<Workmeetinginfo> page = workmeetinginfoService.findPage(new Page<Workmeetinginfo>(request, response, -1), workmeetinginfo);
    		new ExportExcel("workmeeting", Workmeetinginfo.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出workmeeting记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workmeeting/workmeetinginfo/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workmeeting:workmeetinginfo:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<Workmeetinginfo> list = ei.getDataList(Workmeetinginfo.class);
			for (Workmeetinginfo workmeetinginfo : list){
				try{
					workmeetinginfoService.save(workmeetinginfo);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条workmeeting记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条workmeeting记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入workmeeting失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workmeeting/workmeetinginfo/?repage";
    }
	
	/**
	 * 下载导入workmeeting数据模板
	 */
	@RequiresPermissions("workmeeting:workmeetinginfo:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "workmeeting数据导入模板.xlsx";
    		List<Workmeetinginfo> list = Lists.newArrayList(); 
    		new ExportExcel("workmeeting数据", Workmeetinginfo.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workmeeting/workmeetinginfo/?repage";
    }
	
	
	

}