/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workprojectcalc.web;

import com.google.common.collect.Lists;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.modules.workprojectcalc.entity.WorkProjectCalcBuild;
import com.jeeplus.modules.workprojectcalc.service.WorkProjectCalcBuildService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.util.List;

/**
 * 工程计算量底稿Controller
 * @author 马鹏波
 * @version 2018-06-06
 */
@Controller
@RequestMapping(value = "${adminPath}/workprojectcalc/workProjectCalcBuild")
public class WorkProjectCalcBuildController extends BaseController {

	@Autowired
	private WorkProjectCalcBuildService workProjectCalcBuildService;
	
	@ModelAttribute
	public WorkProjectCalcBuild get(@RequestParam(required=false) String id) {
		WorkProjectCalcBuild entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workProjectCalcBuildService.get(id);
		}
		if (entity == null){
			entity = new WorkProjectCalcBuild();
		}
		return entity;
	}
	
	/**
	 * 工程计算量底稿列表页面
	 */
//	@RequiresPermissions("workprojectcalc:workProjectCalcBuild:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkProjectCalcBuild workProjectCalcBuild, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkProjectCalcBuild> page = workProjectCalcBuildService.findPage(new Page<WorkProjectCalcBuild>(request, response), workProjectCalcBuild); 
		model.addAttribute("page", page);
		return "modules/workprojectcalc/workProjectCalcBuildList";
	}

	/**
	 * 查看，增加，编辑工程计算量底稿表单页面
	 */
//	@RequiresPermissions(value={"workprojectcalc:workProjectCalcBuild:view","workprojectcalc:workProjectCalcBuild:add","workprojectcalc:workProjectCalcBuild:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkProjectCalcBuild workProjectCalcBuild, Model model) {
		model.addAttribute("workProjectCalcBuild", workProjectCalcBuild);
		return "modules/workprojectcalc/workProjectCalcBuildForm";
	}

	/**
	 * 保存工程计算量底稿
	 */
//	@RequiresPermissions(value={"workprojectcalc:workProjectCalcBuild:add","workprojectcalc:workProjectCalcBuild:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkProjectCalcBuild workProjectCalcBuild, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workProjectCalcBuild)){
			return form(workProjectCalcBuild, model);
		}
		if(!workProjectCalcBuild.getIsNewRecord()){//编辑表单保存
			WorkProjectCalcBuild t = workProjectCalcBuildService.get(workProjectCalcBuild.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workProjectCalcBuild, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workProjectCalcBuildService.save(t);//保存
		}else{//新增表单保存
			workProjectCalcBuildService.save(workProjectCalcBuild);//保存
		}
		addMessage(redirectAttributes, "保存工程计算量底稿成功");
		return "redirect:"+Global.getAdminPath()+"/workprojectcalc/workProjectCalcBuild/?repage";
	}
	
	/**
	 * 删除工程计算量底稿
	 */
//	@RequiresPermissions("workprojectcalc:workProjectCalcBuild:del")
	@RequestMapping(value = "delete")
	public String delete(WorkProjectCalcBuild workProjectCalcBuild, RedirectAttributes redirectAttributes) {
		workProjectCalcBuildService.delete(workProjectCalcBuild);
		addMessage(redirectAttributes, "删除工程计算量底稿成功");
		return "redirect:"+Global.getAdminPath()+"/workprojectcalc/workProjectCalcBuild/?repage";
	}
	
	/**
	 * 批量删除工程计算量底稿
	 */
//	@RequiresPermissions("workprojectcalc:workProjectCalcBuild:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workProjectCalcBuildService.delete(workProjectCalcBuildService.get(id));
		}
		addMessage(redirectAttributes, "删除工程计算量底稿成功");
		return "redirect:"+Global.getAdminPath()+"/workprojectcalc/workProjectCalcBuild/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
//	@RequiresPermissions("workprojectcalc:workProjectCalcBuild:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkProjectCalcBuild workProjectCalcBuild, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "工程计算量底稿"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkProjectCalcBuild> page = workProjectCalcBuildService.findPage(new Page<WorkProjectCalcBuild>(request, response, -1), workProjectCalcBuild);
    		new ExportExcel("工程计算量底稿", WorkProjectCalcBuild.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出工程计算量底稿记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workprojectcalc/workProjectCalcBuild/?repage";
    }

	/**
	 * 导入Excel数据

	 */
//	@RequiresPermissions("workprojectcalc:workProjectCalcBuild:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkProjectCalcBuild> list = ei.getDataList(WorkProjectCalcBuild.class);
			for (WorkProjectCalcBuild workProjectCalcBuild : list){
				try{
					workProjectCalcBuildService.save(workProjectCalcBuild);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条工程计算量底稿记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条工程计算量底稿记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入工程计算量底稿失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workprojectcalc/workProjectCalcBuild/?repage";
    }
	
	/**
	 * 下载导入工程计算量底稿数据模板
	 */
//	@RequiresPermissions("workprojectcalc:workProjectCalcBuild:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "工程计算量底稿数据导入模板.xlsx";
    		List<WorkProjectCalcBuild> list = Lists.newArrayList(); 
    		new ExportExcel("工程计算量底稿数据", WorkProjectCalcBuild.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workprojectcalc/workProjectCalcBuild/?repage";
    }
	
	
	

}