/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workregularapply.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.modules.sys.dao.OfficeDao;
import com.jeeplus.modules.sys.entity.Office;
import com.jeeplus.modules.sys.utils.UserUtils;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workregularapply.entity.WorkRegularContent;
import com.jeeplus.modules.workregularapply.service.WorkRegularContentService;

/**
 * 鉴定内容码表Controller
 * @author chenyuping
 * @version 2018-07-30
 */
@Controller
@RequestMapping(value = "${adminPath}/workregularapply/workRegularContent")
public class WorkRegularContentController extends BaseController {
	@Autowired
	private HttpServletRequest request;
	@Autowired
	private OfficeDao officeDao;

	@Autowired
	private WorkRegularContentService workRegularContentService;
	
	@ModelAttribute
	public WorkRegularContent get(@RequestParam(required=false) String id) {
		WorkRegularContent entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workRegularContentService.get(id);
		}
		if (entity == null){
			entity = new WorkRegularContent();
		}
		return entity;
	}
	
	/**
	 * 鉴定内容码表列表页面
	 */
	@RequiresPermissions("workregularapply:workRegularContent:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkRegularContent workRegularContent, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkRegularContent> page = workRegularContentService.findPage(new Page<WorkRegularContent>(request, response), workRegularContent); 
		model.addAttribute("page", page);
		return "modules/workregularapply/workRegularContentList";
	}

	/**
	 * 查看，增加，编辑鉴定内容码表表单页面
	 */
	@RequiresPermissions(value={"workregularapply:workRegularContent:view","workregularapply:workRegularContent:add","workregularapply:workRegularContent:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkRegularContent workRegularContent, Model model) {
		model.addAttribute("workRegularContent", workRegularContent);
		String view = "workRegularContentForm";
		String tabId = request.getParameter("tabId");
		Office office=UserUtils.getSelectOffice();

		if(StringUtils.isNotBlank(office.getBranchOffice())){
			Office branchOffice=officeDao.get(office.getBranchOffice());
			workRegularContent.setCompanyName(branchOffice.getName());
		}else{
			workRegularContent.setCompanyName(UserUtils.getSelectCompany().getName());
		}
		if("1".equals(tabId)){
			view = "workRegularContentview";
		}
		return "modules/workregularapply/"+view;
	}

	/**
	 * 保存鉴定内容码表
	 */
	@RequiresPermissions(value={"workregularapply:workRegularContent:add","workregularapply:workRegularContent:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkRegularContent workRegularContent, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workRegularContent)){
			return form(workRegularContent, model);
		}
		workRegularContent.setOfficeId(UserUtils.getSelectOffice().getId());
		workRegularContent.setCompanyId(UserUtils.getSelectCompany().getId());
		if(!workRegularContent.getIsNewRecord()){//编辑表单保存
			WorkRegularContent t = workRegularContentService.get(workRegularContent.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workRegularContent, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workRegularContentService.save(t);//保存
		}else{//新增表单保存
			workRegularContentService.save(workRegularContent);//保存
		}
		addMessage(redirectAttributes, "保存鉴定内容成功");
		return "redirect:"+Global.getAdminPath()+"/workregularapply/workRegularContent/?repage";
	}
	
	/**
	 * 删除鉴定内容码表
	 */
	@RequiresPermissions("workregularapply:workRegularContent:del")
	@RequestMapping(value = "delete")
	public String delete(WorkRegularContent workRegularContent, RedirectAttributes redirectAttributes) {
		workRegularContentService.delete(workRegularContent);
		addMessage(redirectAttributes, "删除鉴定内容码表成功");
		return "redirect:"+Global.getAdminPath()+"/workregularapply/workRegularContent/?repage";
	}
	
	/**
	 * 批量删除鉴定内容码表
	 */
	@RequiresPermissions("workregularapply:workRegularContent:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workRegularContentService.delete(workRegularContentService.get(id));
		}
		addMessage(redirectAttributes, "删除鉴定内容码表成功");
		return "redirect:"+Global.getAdminPath()+"/workregularapply/workRegularContent/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workregularapply:workRegularContent:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkRegularContent workRegularContent, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "鉴定内容码表"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkRegularContent> page = workRegularContentService.findPage(new Page<WorkRegularContent>(request, response, -1), workRegularContent);
    		new ExportExcel("鉴定内容码表", WorkRegularContent.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出鉴定内容码表记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workregularapply/workRegularContent/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workregularapply:workRegularContent:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkRegularContent> list = ei.getDataList(WorkRegularContent.class);
			for (WorkRegularContent workRegularContent : list){
				try{
					workRegularContentService.save(workRegularContent);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条鉴定内容码表记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条鉴定内容码表记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入鉴定内容码表失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workregularapply/workRegularContent/?repage";
    }
	
	/**
	 * 下载导入鉴定内容码表数据模板
	 */
	@RequiresPermissions("workregularapply:workRegularContent:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "鉴定内容码表数据导入模板.xlsx";
    		List<WorkRegularContent> list = Lists.newArrayList(); 
    		new ExportExcel("鉴定内容码表数据", WorkRegularContent.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workregularapply/workRegularContent/?repage";
    }
	
	
	

}